from arduino import Arduino
import time
import csv


b = Arduino('COM3')
write_pin = 3
read_pin = 'A0'

#Declare output pins as a list/tuple
b.output([])

#Setup output file
output_file = open('output.csv', 'wb')
file_writer = csv.writer(output_file)
file_writer.writerow(["Time","Setpoint","Temperature","Voltage"])

#Initialize
startTime = time.time()
lastTime = int(round(time.time() * 1000))
lastErr = 0
errSum = 0
out_min = 0
out_max = 255
out = 0

##################################################
#####    CHANGE CONTROL VARIABLES HERE   #########
##################################################

setpoint = 90
kp = 1
ki = 1
kd = 1

##################################################
##################################################

#Convert Setpoint to voltage
setpoint_V = (((setpoint-32)/1.8)*10 + 500)

print "Initializing..."

#Run Program
try:
    while (True):
        
        #Read
        time.sleep(.5)
        print "---------------"
        print "INPUT"
        print "read: %s" % b.analogRead(read_pin)
        val = int(b.analogRead(read_pin))
        voltage = val*(3300/1024)
        print "voltage: %s" % voltage
        DegC = (voltage - 500)/10
        DegF = DegC*1.8+32
        read_out = 'DegC: ' + str(DegC) + ' DegF: ' + str(DegF)
        print read_out
        

        print "Setpoint: %s" % setpoint
        print "Setpoint Voltage: %s" % setpoint_V

        ###########################################
        #### CONSTRUCT CONTROLLER HERE ############
        ###########################################
        '''
        Available Variables
        Control:
            setpoint
            kp
            ki
            kd
        Input:
            voltage
        Output:
            out
        '''

        #PID Control
        print '\n PID Info:'
        
        #Get delta t
        now = int(round(time.time() * 1000))
        timeChange = now - lastTime
        print ' Delta t: %s' % timeChange
        
        #Compute Error
        error = (setpoint_V - voltage)
        print ' Error: %s' % error
        errSum = errSum + (error * timeChange)
        print ' ErrorSum: %s' % errSum
        dErr = (error - lastErr) / timeChange
        print ' dErr: %s' % dErr

        #PID Output
        out = kp * error + ki * errSum + kd * dErr
        print '\nOUTPUT'
        print 'Out Calculated: %s' % out

        #Save variables for next iteration
        lastErr = error
        lastTime = now


        ###########################################
        #######   END CONTROLLER  #################
        ###########################################

        #Limit output to 255 mV
        if out > out_max:
            out = out_max
        if out < out_min:
            out = out_min
        print 'Out Limited: %s' % out
 

        #Write
        time.sleep(.5)
        b.analogWrite(write_pin, out)
        print ''

        #Output to File
        file_writer.writerow([time.time() - startTime,setpoint,DegF,out])

        '''
        #Digital Write
        b.setHigh(write_pin)
        time.sleep(.1)
        print b.getState(write_pin)
        b.setLow(write_pin)
        print "Blink"
        time.sleep(.1)
        '''

#End Program with Ctrl C
except KeyboardInterrupt:
    print "\nExiting..."
    b.analogWrite(write_pin, 0)
    b.close()
    output_file.close()

